% BHATTACHARYYA(histogram1, histogram2)
% compute the BHATTACHARYYA distance between 2 normalized histograms
% where each histogram is a Nx1 vector
% 
% N.B. both histograms must be normalised
% (i.e. bin values lie in range 0->1 and SUM(bins(i)) = 1
%       for i = {histogram1, histogram2} )

function bdist = bhattacharyya(histogram1, histogram2)

    % get number of bins 
    bins = length(histogram1);
    
    % estimate the bhattacharyya co-efficient
    bcoeff = 0;
    for i=1:bins
        bcoeff = bcoeff + sqrt(histogram1(i) * histogram2(i));
    end
    
    % get the distance between the two distributions as follows
    %bdist = sqrt(1 - bcoeff);      %suggested in http://www.cse.yorku.ca/~kosta/CompVis_Notes/bhattacharyya.pdf
    % Based on the estimation presented in 
    % "Real-Time Tracking of Non-Rigid Objects using Mean Shift"
    %  D. Comaniciu, V. Ramesh & P. Meer (IEEE CVPR 2000, 142-151)
    
    %From Wikipedia
    bdist = -log(bcoeff);
end