%Results of Experiment

clear all; close all; clc;
%load data
load workspace_20topics_4sDocLength_20box_20centers_HOGHOF.mat

%plot histogram
h=figure;
subplot(2,1,1)
norm = NormalizedDocLi_test(length(anomalous)+1:end);
norm = norm(norm>=-13.5);
hist(norm,-13.5:0.5:-9.5)
title('Normalized Likelihood of Normal Test Documents')
subplot(2,1,2)
hist(NormalizedDocLi_test(1:length(anomalous)),-13.5:0.5:-9.5)
title('Normalized Likelihood of Anomalous Test Documents')
print(h,'-dpng',['Histogram_' num2str(Learn.Num_Topics) 'topics_' num2str(doc_length) 'sDocLength_20boxHOGHOF.png'])

%Notations -	
%tfMatrix - tf matrix of full data i.e. total 663 documents of 4 seconds each (including both test and train).
%doc_indices - indices of documents in tfMatrix (i.e. 1 to 663) which are non-anomalous.
%anomalous - indices of documents in tfMatrix (i.e. 1 to 663) which are anomalous.
%trainData and testData - disjoint partitions of tfMatrix
%Actual Anomalous Docs in test Data - First 1 to length(anomalous) docs in testData are anomalous.
%AnomDocs - indices of documents in testData which are PREDICTED as anomalous.

%Detecting Anomalies : Perecision - Recall Curve
thres = min(NormalizedDocLi_test):0.25:max(NormalizedDocLi_test);
upperAnom = length(anomalous);
prec = zeros(numel(thres),1);
rec = zeros(numel(thres),1);
for i=1:numel(thres)
    AnomDocs=[];
    for j=1:numel(NormalizedDocLi_test)
        if NormalizedDocLi_test(j) < thres(i)
            AnomDocs = [AnomDocs j];
        end
    end
    pre = numel(find(AnomDocs<=upperAnom));
    rec(i) = pre/upperAnom;
    prec(i) = pre/numel(AnomDocs);
end

%Statistics of Experiment -
display(' ');
display('Statistics of Experiment is as follows :');
display(['Total Training Documents = ' num2str(size(trainData,2))]);
display(['Total Test Documents = ' num2str(size(testData,2))]);
display(['Threshold for likelihood distinction = ' num2str(thres)]);
display(['# of Anomalous Test Documents = ' num2str(length(anomalous)) '      Identified as Anomalous = ' num2str(length(AnomDocs(AnomDocs<=length(anomalous))))]);
display(['# of Non-Anomalous Test Documents = ' num2str(size(testData,2)-length(anomalous)) '      Identified as Anomalous = ' num2str(length(AnomDocs(AnomDocs>length(anomalous))))]);

%Printing Duration of Anomalous Documents -
 display('Duration of Identified Anomalous Docs (and actually anomalous) :');
 flag=1;
 for i=1:numel(AnomDocs)
     if AnomDocs(i)>length(anomalous) && flag==1
         flag=0;
         display(' ');
         display('Duration of Identified Anomalous Docs (and actually non-anomalous) :');
     end
 	display(['Anomaly suspected between ' num2str((testdoc(AnomDocs(i))-1)*doc_length) 's and ' num2str(testdoc(AnomDocs(i))*doc_length) 's']);
 end
labels = ones(numel(NormalizedDocLi_test),1);
labels(length(anomalous)+1:end) = -1;
%T = 10:0.5:12
%ROC Coordinates -
[X,Y,T,AUC] = perfcurve(labels,-NormalizedDocLi_test,1);
%Recall and Precision values -
[Xpr,Ypr,Tpr,AUCpr] = perfcurve(labels,-NormalizedDocLi_test,1, 'xCrit', 'reca', 'yCrit', 'prec');
figure
plot(X,Y);
figure
plot(Xpr,Ypr);

%Best Threshold 
best_th = -Tpr(find(Ypr==max(Ypr)));     %the one with best precision
if numel(best_th)>1
    best_th = best_th(1);
end
upperAnom = length(anomalous);
AnomDocs=[];
for j=1:numel(NormalizedDocLi_test)
    if NormalizedDocLi_test(j) < best_th
        AnomDocs = [AnomDocs j];
    end
end
display(' ');
display('Some more statistics :');
display(['Total Training Documents = ' num2str(size(trainData,2))]);
display(['Total Test Documents = ' num2str(size(testData,2))]);
display(['Threshold chosen i.e. one with maximum precision = ' num2str(best_th)]);
display(['# of Anomalous Test Documents = ' num2str(length(anomalous)) '      Identified as Anomalous = ' num2str(length(AnomDocs(AnomDocs<=length(anomalous))))]);
display(['# of Non-Anomalous Test Documents = ' num2str(size(testData,2)-length(anomalous)) '      Identified as Anomalous = ' num2str(length(AnomDocs(AnomDocs>length(anomalous))))]);
%display(['Recall = ' num2str(Xpr(find(Ypr==max(Ypr)))) '      Precision = ' num2str(max(Ypr))]);
display(['Area Under Precision Recall Curve Likelihood Model = ' num2str(AUCpr)]);