%This file implements the projection Model. This model finds anomalous
%words in a test documents (localization of anomaly) and on the basis of
%that we decide whether document is anomalous or not (document anomalous
%classification).

%Parameters of Projection Model to be decided empirically -
%k topics
%m nearest docs
%l nearest words
%th threshold

clc
clear all
load workspace_20topics_4sDocLength_20box_20centers_HOGHOF.mat

%Input to Model ===========================================================

%Training Data -
%Pz_d : P(z|d) Matrix
%Pz_w : P(z|w) Matrix
%trainData : words x document : histogram over words of each document
Pz_d=[];
for i=1:size(Pd_z,1)
	probDocument = sum(Pd_z(i,:).*Pz);
    Pz_d(:,i) = (Pd_z(i,:).*Pz) / probDocument;
end
Pz_w=[];
for i=1:size(Pw_z,1)
	probWord = sum(Pw_z(i,:).*Pz);
    Pz_w(:,i) = (Pw_z(i,:).*Pz) / probWord;
end

%Test Data -
%Pz_d_test : P(z|d) Matrix
%Pz_w_test : not required. We use P(z|w) from training data. P(w|z) is same for training and test, but don't know about P(z|w) whether it is samee in train-test or not.
%testData : words x document : histogram over words of each document
Pz_d_test=[];
for i=1:size(Pd_z_test,1)
    probDocument = sum(Pd_z_test(i,:).*Pz_test);
	Pz_d_test(:,i) = Pz_test.*Pd_z_test(i,:) / probDocument;
end

%Output of Model ==========================================================

%anomalyMatrix  : words x testDocs matrix (same in size as testData)
%               : domain = {0,1}
%               : anomalyMatrix(i,j) = 1 if i^th word in j^th test document
%               is anomalous, else 0 if that word is usual.
anomalyMatrix = zeros(size(testData));


%Implementaton of Model ===================================================

%Usualness is -
%   directly propotional to => [m,l]
%   inversely propotional to => [th,th_neighbours,num_neighbours]
%Parameters :
m=100;%      size(trainData,2);       	%4-5 or check in all trainDocs. m nearest docs
l=1;						%l nearest words
th=1;       %an integer b/c count of occurrence of word. this for current "word".
th_neighbours=20;           %Threshold for neighbours
num_neighbours=3;           %Least number of neighbours to have occured earlier, to call current "word" usual.

%Pre-processing step for nearest word analysis :
%We cannot use k-d tree i.e. an efficient implementation of nearest neighbours instead of O(n^2) naive method b/c it is not defined for bhattachary metric.
%nearestWordIndexMatrix : word x word Matrix
%   : i^th row vector is nearest words in increasing order
%   : i.e. for word i, indices of nearest neighbours in order = nearestWordIndexMatrix(i,:)

%nearestWordIndexMatrix = zeros(size(trainData,1),size(trainData,1));
%if (matlabpool('size') == 0)
%    matlabpool open
%end
%parfor word=1:size(trainData,1)
%    wVector = Pz_w(:,word); %topic vector of word
%    distanceFromWords = zeros(size(trainData,1),1);
%    for i=1:size(trainData,1)
%        wiVector = Pz_w(:,i);   %topic vector of i^th word
%        distanceFromWords(i,1) = bhattacharyya(wVector,wiVector);   %Bhattacharya distance b/w wVector & wiVector
%    end
%    [~,nearestWordIndex]=sort(distanceFromWords,'ascend'); %first word in index is nearest
%    
%    nearestWordIndexMatrix(word,:) = nearestWordIndex;
%end
%if (matlabpool('size') > 0)
%    matlabpool close
%end
%save(['nearestWordIndexMatrix_' num2str(Learn.Num_Topics) 'topics_' num2str(windowSize) 'box.mat'],'nearestWordIndexMatrix');
%load(['nearestWordIndexMatrix_' num2str(Learn.Num_Topics) 'topics_' num2str(windowSize) 'box' num2str(flowCentres) 'centers_HOGHOF.mat']);

%Algorithm Code :
for ts=1:size(testData,2)
    
    %Step 1: Nearest Docs using Bhatt. distance
    dts = Pz_d_test(:,ts);   %topic vector of test document
    distanceFromTrainDocs = zeros(size(trainData,2),1);
    for tr=1:size(trainData,2)
        dtr = Pz_d(:,tr);   %topic vector of train document
        distanceFromTrainDocs(tr,1) = bhattacharyya(dts,dtr);  %Bhattacharya distance b/w dts & dtr
    end
    [~,nearestDocIndex]=sort(distanceFromTrainDocs,'ascend');  %first topic in index is nearest, followed by others in order
    
    %Step 2: Cumulative Word Histogram of m nearest train docs
    cumulativeHistogram = zeros(size(trainData,1),1);
    for i=1:m
        cumulativeHistogram = cumulativeHistogram + trainData(:,nearestDocIndex(i));
    end

    %Step 3: Word by Word analysis for test document
    for word=1:size(testData,1)
        
        %Step 3a: Word does not occur in test
        if testData(word,ts)==0
            anomalyMatrix(word,ts)=0;       %word in usual/non-existent
            continue
        end
        %size(FramesForWords{word}{testdoc(ts)})
        %Step 3b: Nearest Words to this word using Bhattacharya distance.
        %nearestWordIndex = nearestWordIndexMatrix(word,:);
        nearestWordIndex(1) = word; %l=1
		
        %Step 3c: Check frequency of l nearest words in cumulative histogram
        flag = 0;
        for i=1:l
            if ( cumulativeHistogram(nearestWordIndex(i)) > th)
                flag = 100;
            end
        end
        
        %Get nearest words in location to the current "word" i.e. words in
        %upper-down-left-right-upLeft-upRight-downLeft-downRight cell with 
        %same (hoghof,size) values.
        %Image is h=288xw=360:14x18. For each value of i: 18**flowCentres*2 values ; for each i,j: flowCentres*2 values ; for each i,j,hoghof: 2 values.
        %Formula = ((i/windowSize)*18+(j/windowSize))*(flowCentres*2)  + opticalFlow*2 + size + 1
        nearestWordIndex(2) = word + (flowCentres*2);    %towards Right
        nearestWordIndex(3) = word - (flowCentres*2);    %towards Left
        nearestWordIndex(4) = word + (18*flowCentres*2); %towards Down
        nearestWordIndex(5) = word - (18*flowCentres*2); %towards Up
        nearestWordIndex(6) = word + (flowCentres*2) + (18*flowCentres*2);  %towards RightDown
        nearestWordIndex(7) = word - (flowCentres*2) + (18*flowCentres*2);  %towards LeftDown
        nearestWordIndex(8) = word + (flowCentres*2) - (18*flowCentres*2);  %towards RightUp
        nearestWordIndex(9) = word - (flowCentres*2) - (18*flowCentres*2);  %towards LeftUp
        for i=2:length(nearestWordIndex)
            % checking boundary condition
            if nearestWordIndex(i)<1 || nearestWordIndex(i)>length(cumulativeHistogram)
                continue
            end
            if ( cumulativeHistogram(nearestWordIndex(i)) > th_neighbours)
                flag = flag+1;
            end
        end
        
        %Step 3d: Anomalous or not
        if flag>=num_neighbours
            anomalyMatrix(word,ts)=0;       %word is usual/existent
        else
            anomalyMatrix(word,ts)=1;       %word is anomalous/existent
        end
        
        %Word Analysis complete
    end
    
    %Projection Model completed.
end

anomalousWordsperDoc = sum(anomalyMatrix); %sum columns individually

anomalousTestDocs = 1:length(anomalous);   %All anomalous docs are placed on top in testData
usualTestDocs = (length(anomalous)+1):size(testData,2);
display('\n\nAnomalos words in Anomalous Docs :');
sort(anomalousWordsperDoc(anomalousTestDocs))
display('\n\nAnomalos words in Usual Docs :');
sort(anomalousWordsperDoc(usualTestDocs))

labels = zeros(size(testData,2),1);
labels(anomalousTestDocs) = 1;
[Xpr,Ypr,Tpr,AUCpr] = perfcurve(labels,anomalousWordsperDoc',1, 'xCrit', 'reca', 'yCrit', 'prec');
[X,Y,T,AUC] = perfcurve(labels,anomalousWordsperDoc,1);
h=figure;
plot(Xpr,Ypr);
ylim([0 1]);
xlabel('Recall');
ylabel('Precision');
title('Precision Recall curve for Projection Model');
print(h,'-dpng',['ProjectionModel_' num2str(Learn.Num_Topics) 'topics_' num2str(windowSize) 'box.png']);
display(['Area Under Precision Recall Curve = ' num2str(AUCpr)]);
truncatedPrecision = Ypr(find(Xpr>0.35));
bestPrecision = max(truncatedPrecision);
bestThres = Tpr(find(Xpr>0.35 & Ypr==bestPrecision));
bestThres = bestThres(end);
bestRecall = Xpr(find(Tpr==bestThres));
display(['Optimal Recall = ' num2str(bestRecall(end)) '  Optimal Precision = ' num2str(bestPrecision(end))]);
%save(['FullProjModelWorkspace_' num2str(Learn.Num_Topics) 'topics_' num2str(doc_length) 'sDocLength_' num2str(windowSize) 'box.mat']);
%clear all;

